@@ -6,8 +6,11 @@
  * @subpackage Press_This
  */
 
+define('IFRAME_REQUEST' , true);
+
 /** WordPress Administration Bootstrap */
 require_once('./admin.php');
+
 header('Content-Type: ' . get_option('html_type') . '; charset=' . get_option('blog_charset'));
 
 if ( ! current_user_can('edit_posts') )
@@ -23,18 +26,17 @@ if ( ! current_user_can('edit_posts') )
  * @return int Post ID
  */
 function press_it() {
-	// define some basic variables
-	$quick['post_status'] = 'draft'; // set as draft first
-	$quick['post_category'] = isset($_POST['post_category']) ? $_POST['post_category'] : null;
-	$quick['tax_input'] = isset($_POST['tax_input']) ? $_POST['tax_input'] : null;
-	$quick['post_title'] = ( trim($_POST['title']) != '' ) ? $_POST['title'] : '  ';
-	$quick['post_content'] = isset($_POST['post_content']) ? $_POST['post_content'] : '';
 
-	// insert the post with nothing in it, to get an ID
-	$post_ID = wp_insert_post($quick, true);
-	if ( is_wp_error($post_ID) )
-		wp_die($post_ID);
+	$post = get_default_post_to_edit();
+	$post = get_object_vars($post);
+	$post_ID = $post['ID'] = (int) $_POST['post_id'];
 
+	if ( !current_user_can('edit_post', $post_ID) )
+		wp_die(__('You are not allowed to edit this post.'));
+
+	$post['post_category'] = isset($_POST['post_category']) ? $_POST['post_category'] : '';
+	$post['tax_input'] = isset($_POST['tax_input']) ? $_POST['tax_input'] : '';
+	$post['post_title'] = isset($_POST['title']) ? $_POST['title'] : '';
 	$content = isset($_POST['content']) ? $_POST['content'] : '';
 
 	$upload = false;
@@ -52,26 +54,40 @@ function press_it() {
 		}
 	}
 	// set the post_content and status
-	$quick['post_status'] = isset($_POST['publish']) ? 'publish' : 'draft';
-	$quick['post_content'] = $content;
+	$post['post_content'] = $content;
+	if ( isset( $_POST['publish'] ) && current_user_can( 'publish_posts' ) )
+		$post['post_status'] = 'publish';
+	elseif ( isset( $_POST['review'] ) )
+		$post['post_status'] = 'pending';
+	else
+		$post['post_status'] = 'draft';
+
 	// error handling for media_sideload
 	if ( is_wp_error($upload) ) {
 		wp_delete_post($post_ID);
 		wp_die($upload);
 	} else {
-		$quick['ID'] = $post_ID;
-		wp_update_post($quick);
+		// Post formats
+		if ( isset( $_POST['post_format'] ) ) {
+			if ( current_theme_supports( 'post-formats', $_POST['post_format'] ) )
+				set_post_format( $post_ID, $_POST['post_format'] );
+			elseif ( '0' == $_POST['post_format'] )
+				set_post_format( $post_ID, false );
+		}
+
+		$post_ID = wp_update_post($post);
 	}
+
 	return $post_ID;
 }
 
 // For submitted posts.
 if ( isset($_REQUEST['action']) && 'post' == $_REQUEST['action'] ) {
 	check_admin_referer('press-this');
-	$post_ID = press_it();
-	$posted =  $post_ID;
+	$posted = $post_ID = press_it();
 } else {
-	$post_ID = 0;
+	$post = get_default_post_to_edit('post', true);
+	$post_ID = $post->ID;
 }
 
 // Set Variables
@@ -110,7 +126,7 @@ if ( !empty($_REQUEST['ajax']) ) {
 			<div class="postbox">
 				<h2><label for="embed-code"><?php _e('Embed Code') ?></label></h2>
 				<div class="inside">
-					<textarea name="embed-code" id="embed-code" rows="8" cols="40"><?php echo wp_htmledit_pre( $selection ); ?></textarea>
+					<textarea name="embed-code" id="embed-code" rows="8" cols="40"><?php echo esc_textarea( $selection ); ?></textarea>
 					<p id="options"><a href="#" class="select button"><?php _e('Insert Video'); ?></a> <a href="#" class="close button"><?php _e('Cancel'); ?></a></p>
 				</div>
 			</div>
@@ -123,19 +139,19 @@ if ( !empty($_REQUEST['ajax']) ) {
 					tb_remove();
 				});
 				jQuery('.select').click(function() {
-					image_selector();
+					image_selector(this);
 				});
 				/* ]]> */
 			</script>
-			<h3 class="tb"><label for="this_photo_description"><?php _e('Description') ?></label></h3>
+			<h3 class="tb"><label for="tb_this_photo_description"><?php _e('Description') ?></label></h3>
 			<div class="titlediv">
 				<div class="titlewrap">
-					<input id="this_photo_description" name="photo_description" class="tbtitle text" onkeypress="if(event.keyCode==13) image_selector();" value="<?php echo esc_attr($title);?>"/>
+					<input id="tb_this_photo_description" name="photo_description" class="tb_this_photo_description tbtitle text" onkeypress="if(event.keyCode==13) image_selector(this);" value="<?php echo esc_attr($title);?>"/>
 				</div>
 			</div>
 
 			<p class="centered">
-				<input type="hidden" name="this_photo" value="<?php echo esc_attr($image); ?>" id="this_photo" />
+				<input type="hidden" name="this_photo" value="<?php echo esc_attr($image); ?>" id="tb_this_photo" class="tb_this_photo" />
 				<a href="#" class="select">
 					<img src="<?php echo esc_url($image); ?>" alt="<?php echo esc_attr(__('Click to insert.')); ?>" title="<?php echo esc_attr(__('Click to insert.')); ?>" />
 				</a>
@@ -143,34 +159,6 @@ if ( !empty($_REQUEST['ajax']) ) {
 
 			<p id="options"><a href="#" class="select button"><?php _e('Insert Image'); ?></a> <a href="#" class="cancel button"><?php _e('Cancel'); ?></a></p>
 			<?php break;
-
-		case 'photo_thickbox_url': ?>
-			<script type="text/javascript" charset="utf-8">
-				/* <![CDATA[ */
-				jQuery('.cancel').click(function() {
-					tb_remove();
-				});
-
-				jQuery('.select').click(function() {
-					image_selector();
-				});
-				/* ]]> */
-			</script>
-			<h3 class="tb"><label for="this_photo"><?php _e('URL') ?></label></h3>
-			<div class="titlediv">
-				<div class="titlewrap">
-					<input id="this_photo" name="this_photo" class="tbtitle text" onkeypress="if(event.keyCode==13) image_selector();" />
-				</div>
-			</div>
-			<h3 class="tb"><label for="photo_description"><?php _e('Description') ?></label></h3>
-			<div id="titlediv">
-				<div class="titlewrap">
-					<input id="this_photo_description" name="photo_description" class="tbtitle text" onkeypress="if(event.keyCode==13) image_selector();" value="<?php echo esc_attr($title);?>"/>
-				</div>
-			</div>
-
-			<p id="options"><a href="#" class="select"><?php _e('Insert Image'); ?></a> | <a href="#" class="cancel"><?php _e('Cancel'); ?></a></p>
-			<?php break;
 	case 'photo_images':
 		/**
 		 * Retrieve all image URLs from given URI.
@@ -204,7 +192,7 @@ if ( !empty($_REQUEST['ajax']) ) {
 						$src = 'http://'.str_replace('//','/', $host['host'].'/'.$src);
 					else
 						$src = 'http://'.str_replace('//','/', $host['host'].'/'.dirname($host['path']).'/'.$src);
-				$sources[] = esc_attr($src);
+				$sources[] = esc_url($src);
 			}
 			return "'" . implode("','", $sources) . "'";
 		}
@@ -219,22 +207,22 @@ if ( !empty($_REQUEST['ajax']) ) {
 		if(photostorage == false) {
 		var my_src = eval(
 			jQuery.ajax({
-		   		type: "GET",
-		   		url: "<?php echo esc_url($_SERVER['PHP_SELF']); ?>",
+				type: "GET",
+				url: "<?php echo esc_url($_SERVER['PHP_SELF']); ?>",
 				cache : false,
 				async : false,
-		   		data: "ajax=photo_images&u=<?php echo urlencode($url); ?>",
+				data: "ajax=photo_images&u=<?php echo urlencode($url); ?>",
 				dataType : "script"
 			}).responseText
 		);
 		if(my_src.length == 0) {
 			var my_src = eval(
 				jQuery.ajax({
-		   			type: "GET",
-		   			url: "<?php echo esc_url($_SERVER['PHP_SELF']); ?>",
+					type: "GET",
+					url: "<?php echo esc_url($_SERVER['PHP_SELF']); ?>",
 					cache : false,
 					async : false,
-		   			data: "ajax=photo_images&u=<?php echo urlencode($url); ?>",
+					data: "ajax=photo_images&u=<?php echo urlencode($url); ?>",
 					dataType : "script"
 				}).responseText
 			);
@@ -282,42 +270,41 @@ if ( !empty($_REQUEST['ajax']) ) {
 			return false;
 		}
 
-		function image_selector() {
+		function image_selector(el) {
+			var desc, src, parent = jQuery(el).closest('#photo-add-url-div');
+
+			if ( parent.length ) {
+				desc = parent.find('input.tb_this_photo_description').val() || '';
+				src = parent.find('input.tb_this_photo').val() || ''
+			} else {
+				desc = jQuery('#tb_this_photo_description').val() || '';
+				src = jQuery('#tb_this_photo').val() || ''
+			}
+
 			tb_remove();
-			desc = jQuery('#this_photo_description').val();
-			src = jQuery('#this_photo').val();
 			pick(src, desc);
 			jQuery('#extra-fields').hide();
 			jQuery('#extra-fields').html('');
 			return false;
 		}
-			jQuery('#extra-fields').html('<div class="postbox"><h2>Add Photos <small id="photo_directions">(<?php _e("click images to select") ?>)</small></h2><ul class="actions"><li><a href="#" id="photo-add-url" class="thickbox button"><?php _e("Add from URL") ?> +</a></li></ul><div class="inside"><div class="titlewrap"><div id="img_container"></div></div><p id="options"><a href="#" class="close button"><?php _e('Cancel'); ?></a><a href="#" class="refresh button"><?php _e('Refresh'); ?></a></p></div>');
-			jQuery('#img_container').html(strtoappend);
+
+		jQuery('#extra-fields').html('<div class="postbox"><h2><?php _e( 'Add Photos' ); ?> <small id="photo_directions">(<?php _e("click images to select") ?>)</small></h2><ul class="actions"><li><a href="#" id="photo-add-url" class="button"><?php _e("Add from URL") ?> +</a></li></ul><div class="inside"><div class="titlewrap"><div id="img_container"></div></div><p id="options"><a href="#" class="close button"><?php _e('Cancel'); ?></a><a href="#" class="refresh button"><?php _e('Refresh'); ?></a></p></div>');
+		jQuery('#img_container').html(strtoappend);
 		<?php break;
 }
 die;
 }
 
-?>
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml" <?php do_action('admin_xml_ns'); ?> <?php language_attributes(); ?>>
-<head>
-	<meta http-equiv="Content-Type" content="<?php bloginfo('html_type'); ?>; charset=<?php echo get_option('blog_charset'); ?>" />
-	<title><?php _e('Press This') ?></title>
-
-<?php
-	add_thickbox();
-	wp_enqueue_style( 'press-this' );
-	wp_enqueue_style( 'press-this-ie');
 	wp_enqueue_style( 'colors' );
 	wp_enqueue_script( 'post' );
-	wp_enqueue_script( 'editor' );
+	_wp_admin_html_begin();
 ?>
+<title><?php _e('Press This') ?></title>
 <script type="text/javascript">
 //<![CDATA[
 addLoadEvent = function(func){if(typeof jQuery!="undefined")jQuery(document).ready(func);else if(typeof wpOnload!='function'){wpOnload=func;}else{var oldonload=wpOnload;wpOnload=function(){oldonload();func();}}};
 var userSettings = {'url':'<?php echo SITECOOKIEPATH; ?>','uid':'<?php if ( ! isset($current_user) ) $current_user = wp_get_current_user(); echo $current_user->ID; ?>','time':'<?php echo time() ?>'};
-var ajaxurl = '<?php echo admin_url('admin-ajax.php'); ?>', pagenow = 'press-this';
+var ajaxurl = '<?php echo admin_url('admin-ajax.php'); ?>', pagenow = 'press-this', isRtl = <?php echo (int) is_rtl(); ?>;
 var photostorage = false;
 //]]>
 </script>
@@ -326,18 +313,34 @@ var photostorage = false;
 	do_action('admin_print_styles');
 	do_action('admin_print_scripts');
 	do_action('admin_head');
-
-	if ( user_can_richedit() )
-		wp_tiny_mce( true, array( 'height' => '370' ) );
 ?>
+	<style type="text/css">
+	#message {
+		margin: 10px 0;
+	}
+	#title,
+	.press-this #wphead {
+		margin-left: 0;
+		margin-right: 0;
+	}
+	.rtl.press-this #header-logo,
+	.rtl.press-this #wphead h1 {
+		float: right;
+	}
+	</style>
 	<script type="text/javascript">
+	var wpActiveEditor = 'content';
+
 	function insert_plain_editor(text) {
-		edCanvas = document.getElementById('content');
-		edInsertContent(edCanvas, text);
+		if ( typeof(QTags) != 'undefined' )
+			QTags.insertContent(text);
 	}
 	function set_editor(text) {
-		if ( '' == text || '<p></p>' == text ) text = '<p><br /></p>';
-		if ( tinyMCE.activeEditor ) tinyMCE.execCommand('mceSetContent', false, text);
+		if ( '' == text || '<p></p>' == text )
+			text = '<p><br /></p>';
+
+		if ( tinyMCE.activeEditor )
+			tinyMCE.execCommand('mceSetContent', false, text);
 	}
 	function insert_editor(text) {
 		if ( '' != text && tinyMCE.activeEditor && ! tinyMCE.activeEditor.isHidden()) {
@@ -349,7 +352,6 @@ var photostorage = false;
 	function append_editor(text) {
 		if ( '' != text && tinyMCE.activeEditor && ! tinyMCE.activeEditor.isHidden()) {
 			tinyMCE.execCommand('mceSetContent', false, tinyMCE.activeEditor.getContent({format : 'raw'}) + '<p>' + text + '</p>');
-			tinyMCE.execCommand('mceCleanup');
 		} else {
 			insert_plain_editor(text);
 		}
@@ -388,19 +390,30 @@ var photostorage = false;
 				function setup_photo_actions() {
 					jQuery('.close').click(function() {
 						jQuery('#extra-fields').hide();
+						jQuery('body').append( jQuery('#photo-add-url-div') );
 						jQuery('#extra-fields').html('');
 					});
 					jQuery('.refresh').click(function() {
 						photostorage = false;
+						jQuery('body').append( jQuery('#photo-add-url-div') );
 						show('photo');
 					});
-					jQuery('#photo-add-url').attr('href', '?ajax=photo_thickbox_url&height=200&width=500');
-					tb_init('#extra-fields .thickbox');
+					jQuery('#photo-add-url').click(function(){
+						var container = jQuery('#img_container');
+
+						if ( container.children('#photo-add-url-div:visible').length ) {
+							container.children('a').show();
+							jQuery('#photo-add-url-div').hide();
+						} else {
+							container.children('a').hide();
+							container.append( jQuery('#photo-add-url-div').show() );
+						}
+					});
 					jQuery('#waiting').hide();
 					jQuery('#extra-fields').show();
 				}
-				jQuery('#extra-fields').before('<div id="waiting"><img src="<?php echo esc_url( admin_url( 'images/wpspin_light.gif' ) ); ?>" alt="" /> <?php echo esc_js( __( 'Loading...' ) ); ?></div>');
 
+				jQuery('#waiting').show();
 				if(photostorage == false) {
 					jQuery.ajax({
 						type: "GET",
@@ -424,9 +437,9 @@ var photostorage = false;
 	}
 	jQuery(document).ready(function($) {
 		//resize screen
-		window.resizeTo(720,540);
+		window.resizeTo(720,580);
 		// set button actions
-    	jQuery('#photo_button').click(function() { show('photo'); return false; });
+		jQuery('#photo_button').click(function() { show('photo'); return false; });
 		jQuery('#video_button').click(function() { show('video'); return false; });
 		// auto select
 		<?php if ( preg_match("/youtube\.com\/watch/i", $url) ) { ?>
@@ -445,44 +458,57 @@ var photostorage = false;
 	});
 </script>
 </head>
-<body class="press-this wp-admin">
-<div id="wphead"></div>
+<body class="press-this wp-admin<?php if ( is_rtl() ) echo ' rtl'; ?>">
 <form action="press-this.php?action=post" method="post">
 <div id="poststuff" class="metabox-holder">
-	<div id="side-info-column">
+	<div id="side-sortables" class="press-this-sidebar">
 		<div class="sleeve">
-			<h1 id="viewsite"><a href="<?php echo get_option('home'); ?>/" target="_blank"><?php bloginfo('name'); ?> &rsaquo; <?php _e('Press This') ?></a></span></h1>
-
 			<?php wp_nonce_field('press-this') ?>
 			<input type="hidden" name="post_type" id="post_type" value="text"/>
 			<input type="hidden" name="autosave" id="autosave" />
 			<input type="hidden" id="original_post_status" name="original_post_status" value="draft" />
 			<input type="hidden" id="prev_status" name="prev_status" value="draft" />
+			<input type="hidden" id="post_id" name="post_id" value="<?php echo (int) $post_ID; ?>" />
 
 			<!-- This div holds the photo metadata -->
 			<div class="photolist"></div>
 
-			<div id="submitdiv" class="stuffbox">
-				<div class="handlediv" title="<?php _e( 'Click to toggle' ); ?>">
-					<br/>
-				</div>
-				<h3><?php _e('Publish') ?></h3>
+			<div id="submitdiv" class="postbox">
+				<div class="handlediv" title="<?php esc_attr_e( 'Click to toggle' ); ?>"><br /></div>
+				<h3 class="hndle"><?php _e('Press This') ?></h3>
 				<div class="inside">
-					<p>
-						<input class="button" type="submit" name="draft" value="<?php esc_attr_e('Save Draft') ?>" id="save" />
-						<?php if ( current_user_can('publish_posts') ) { ?>
-							<input class="button-primary" type="submit" name="publish" value="<?php esc_attr_e('Publish') ?>" id="publish" />
-						<?php } else { ?>
-							<br /><br /><input class="button-primary" type="submit" name="review" value="<?php esc_attr_e('Submit for Review') ?>" id="review" />
-						<?php } ?>
+					<p id="publishing-actions">
+					<?php
+						submit_button( __( 'Save Draft' ), 'button', 'draft', false, array( 'id' => 'save' ) );
+						if ( current_user_can('publish_posts') ) {
+							submit_button( __( 'Publish' ), 'primary', 'publish', false );
+						} else {
+							echo '<br /><br />';
+							submit_button( __( 'Submit for Review' ), 'primary', 'review', false );
+						} ?>
 						<img src="<?php echo esc_url( admin_url( 'images/wpspin_light.gif' ) ); ?>" alt="" id="saving" style="display:none;" />
 					</p>
+					<?php if ( current_theme_supports( 'post-formats' ) && post_type_supports( 'post', 'post-formats' ) ) :
+							$post_formats = get_theme_support( 'post-formats' );
+							if ( is_array( $post_formats[0] ) ) :
+								$default_format = get_option( 'default_post_format', '0' );
+						?>
+					<p>
+						<label for="post_format"><?php _e( 'Post Format:' ); ?>
+						<select name="post_format" id="post_format">
+							<option value="0"><?php _ex( 'Standard', 'Post format' ); ?></option>
+						<?php foreach ( $post_formats[0] as $format ): ?>
+							<option<?php selected( $default_format, $format ); ?> value="<?php echo esc_attr( $format ); ?>"> <?php echo esc_html( get_post_format_string( $format ) ); ?></option>
+						<?php endforeach; ?>
+						</select></label>
+					</p>
+					<?php endif; endif; ?>
 				</div>
 			</div>
 
 			<?php $tax = get_taxonomy( 'category' ); ?>
 			<div id="categorydiv" class="postbox">
-				<div class="handlediv" title="<?php _e( 'Click to toggle' ); ?>"><br /></div>
+				<div class="handlediv" title="<?php esc_attr_e( 'Click to toggle' ); ?>"><br /></div>
 				<h3 class="hndle"><?php _e('Categories') ?></h3>
 				<div class="inside">
 				<div id="taxonomy-category" class="categorydiv">
@@ -531,31 +557,46 @@ var photostorage = false;
 				</div>
 			</div>
 
-			<div id="tagsdiv-post_tag" class="stuffbox" >
-				<div class="handlediv" title="<?php _e( 'Click to toggle' ); ?>">
-					<br/>
-				</div>
-				<h3><span><?php _e('Post Tags'); ?></span></h3>
+			<div id="tagsdiv-post_tag" class="postbox">
+				<div class="handlediv" title="<?php esc_attr_e( 'Click to toggle' ); ?>"><br /></div>
+				<h3><span><?php _e('Tags'); ?></span></h3>
 				<div class="inside">
 					<div class="tagsdiv" id="post_tag">
-						<p class="jaxtag">
-							<label class="screen-reader-text" for="newtag"><?php _e('Post Tags'); ?></label>
+						<div class="jaxtag">
+							<label class="screen-reader-text" for="newtag"><?php _e('Tags'); ?></label>
 							<input type="hidden" name="tax_input[post_tag]" class="the-tags" id="tax-input[post_tag]" value="" />
 							<div class="ajaxtag">
 								<input type="text" name="newtag[post_tag]" class="newtag form-input-tip" size="16" autocomplete="off" value="" />
 								<input type="button" class="button tagadd" value="<?php esc_attr_e('Add'); ?>" tabindex="3" />
 							</div>
-						</p>
+						</div>
 						<div class="tagchecklist"></div>
 					</div>
-					<p class="tagcloud-link"><a href="#titlediv" class="tagcloud-link" id="link-post_tag"><?php _e('Choose from the most used tags in Post Tags'); ?></a></p>
+					<p class="tagcloud-link"><a href="#titlediv" class="tagcloud-link" id="link-post_tag"><?php _e('Choose from the most used tags'); ?></a></p>
 				</div>
 			</div>
 		</div>
 	</div>
 	<div class="posting">
-		<?php if ( isset($posted) && intval($posted) ) { $post_ID = intval($posted); ?>
-		<div id="message" class="updated"><p><strong><?php _e('Your post has been saved.'); ?></strong> <a onclick="window.opener.location.replace(this.href); window.close();" href="<?php echo get_permalink( $post_ID); ?>"><?php _e('View post'); ?></a> | <a href="<?php echo get_edit_post_link( $post_ID ); ?>" onclick="window.opener.location.replace(this.href); window.close();"><?php _e('Edit Post'); ?></a> | <a href="#" onclick="window.close();"><?php _e('Close Window'); ?></a></p></div>
+
+		<div id="wphead">
+			<img id="header-logo" src="<?php echo esc_url( includes_url( 'images/blank.gif' ) ); ?>" alt="" width="16" height="16" />
+			<h1 id="site-heading">
+				<a href="<?php echo get_option('home'); ?>/" target="_blank">
+					<span id="site-title"><?php bloginfo('name'); ?></span>
+				</a>
+			</h1>
+		</div>
+
+		<?php
+		if ( isset($posted) && intval($posted) ) {
+			$post_ID = intval($posted); ?>
+			<div id="message" class="updated">
+			<p><strong><?php _e('Your post has been saved.'); ?></strong>
+			<a onclick="window.opener.location.replace(this.href); window.close();" href="<?php echo get_permalink($post_ID); ?>"><?php _e('View post'); ?></a>
+			| <a href="<?php echo get_edit_post_link( $post_ID ); ?>" onclick="window.opener.location.replace(this.href); window.close();"><?php _e('Edit Post'); ?></a>
+			| <a href="#" onclick="window.close();"><?php _e('Close Window'); ?></a></p>
+			</div>
 		<?php } ?>
 
 		<div id="titlediv">
@@ -564,47 +605,69 @@ var photostorage = false;
 			</div>
 		</div>
 
+		<div id="waiting" style="display: none"><img src="<?php echo esc_url( admin_url( 'images/wpspin_light.gif' ) ); ?>" alt="" /> <?php esc_html_e( 'Loading...' ); ?></div>
+
 		<div id="extra-fields" style="display: none"></div>
 
 		<div class="postdivrich">
-			<ul id="actions" class="actions">
+		<?php
 
-				<li id="photo_button">
-					Add: <?php if ( current_user_can('upload_files') ) { ?><a title="<?php _e('Insert an Image'); ?>" href="#">
-<img alt="<?php _e('Insert an Image'); ?>" src="<?php echo esc_url( admin_url( 'images/media-button-image.gif?ver=20100531' ) ); ?>"/></a>
-					<?php } ?>
-				</li>
-				<li id="video_button">
-					<a title="<?php _e('Embed a Video'); ?>" href="#"><img alt="<?php _e('Embed a Video'); ?>" src="<?php echo esc_url( admin_url( 'images/media-button-video.gif?ver=20100531' ) ); ?>"/></a>
-				</li>
-				<?php if ( user_can_richedit() ) { ?>
-				<li id="switcher">
-					<?php wp_print_scripts( 'quicktags' ); ?>
-					<?php add_filter('the_editor_content', 'wp_richedit_pre'); ?>
-					<a id="edButtonHTML" onclick="switchEditors.go('content', 'html');"><?php _e('HTML'); ?></a>
-					<a id="edButtonPreview" class="active" onclick="switchEditors.go('content', 'tinymce');"><?php _e('Visual'); ?></a>
-					<div class="zerosize"><input accesskey="e" type="button" onclick="switchEditors.go('content')" /></div>
-				</li>
-				<?php } ?>
-			</ul>
-			<div id="quicktags"></div>
-			<div class="editor-container">
-				<textarea name="content" id="content" style="width:100%;" class="theEditor" rows="15"><?php
-					if ( $selection )
-						echo wp_richedit_pre($selection);
-					if ( $url ) {
-						echo '<p>';
-						if ( $selection )
-							_e('via ');
-						printf( "<a href='%s'>%s</a>.</p>", esc_url( $url ), esc_html( $title ) );
-					}
-				?></textarea>
-			</div>
+		$editor_settings = array(
+			'teeny' => true,
+			'textarea_rows' => '15'
+		);
+
+		$content = '';
+		if ( $selection )
+			$content .=  $selection;
+
+		if ( $url ) {
+			$content .= '<p>';
+
+			if ( $selection )
+				$content .= __('via ');
+
+			$content .= sprintf( "<a href='%s'>%s</a>.</p>", esc_url( $url ), esc_html( $title ) );
+		}
+
+		remove_action( 'media_buttons', 'media_buttons' );
+		add_action( 'media_buttons', 'press_this_media_buttons' );
+		function press_this_media_buttons() {
+			_e( 'Add:' );
+
+			if ( current_user_can('upload_files') ) {
+				?>
+				<a id="photo_button" title="<?php esc_attr_e('Insert an Image'); ?>" href="#">
+				<img alt="<?php esc_attr_e('Insert an Image'); ?>" src="<?php echo esc_url( admin_url( 'images/media-button-image.gif?ver=20100531' ) ); ?>"/></a>
+				<?php
+			}
+			?>
+			<a id="video_button" title="<?php esc_attr_e('Embed a Video'); ?>" href="#"><img alt="<?php esc_attr_e('Embed a Video'); ?>" src="<?php echo esc_url( admin_url( 'images/media-button-video.gif?ver=20100531' ) ); ?>"/></a>
+			<?php
+		}
+
+		wp_editor( $content, 'content', $editor_settings );
+
+		?>
 		</div>
 	</div>
 </div>
 </form>
-<?php do_action('admin_print_footer_scripts'); ?>
+<div id="photo-add-url-div" style="display:none;">
+	<table><tr>
+	<td><label for="this_photo"><?php _e('URL') ?></label></td>
+	<td><input type="text" id="this_photo" name="this_photo" class="tb_this_photo text" onkeypress="if(event.keyCode==13) image_selector(this);" /></td>
+	</tr><tr>
+	<td><label for="this_photo_description"><?php _e('Description') ?></label></td>
+	<td><input type="text" id="this_photo_description" name="photo_description" class="tb_this_photo_description text" onkeypress="if(event.keyCode==13) image_selector(this);" value="<?php echo esc_attr($title);?>"/></td>
+	</tr><tr>
+	<td><input type="button" class="button" onclick="image_selector(this)" value="<?php esc_attr_e('Insert Image'); ?>" /></td>
+	</tr></table>
+</div>
+<?php
+do_action('admin_footer');
+do_action('admin_print_footer_scripts');
+?>
 <script type="text/javascript">if(typeof wpOnload=='function')wpOnload();</script>
 </body>
 </html>
