@@ -719,13 +719,13 @@ class Snoopy
 							chr(176),
 							chr(39),
 							chr(128),
-							"",
-							"",
-							"",
-							"",
-							"",
-							"",
-							"",
+							chr(0xE4), // ANSI &auml;
+							chr(0xF6), // ANSI &ouml;
+							chr(0xFC), // ANSI &uuml;
+							chr(0xC4), // ANSI &Auml;
+							chr(0xD6), // ANSI &Ouml;
+							chr(0xDC), // ANSI &Uuml;
+							chr(0xDF), // ANSI &szlig;
 						);
 
 		$text = preg_replace($search,$replace,$document);
