@@ -2,6 +2,8 @@
 /**
  * These functions are needed to load WordPress.
  *
+ * @internal This file must be parsable by PHP4.
+ *
  * @package WordPress
  */
 
@@ -95,6 +97,9 @@ function wp_fix_server_vars() {
  *
  * Dies if requirements are not met.
  *
+ * This function must be able to work without a complete environment set up. In wp-load.php, for
+ * example, WP_CONTENT_DIR is defined and version.php is included before this function is called.
+ *
  * @access private
  * @since 3.0.0
  */
@@ -161,7 +166,7 @@ function wp_maintenance() {
 	header( 'Content-Type: text/html; charset=utf-8' );
 	header( 'Retry-After: 600' );
 ?>
-	<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+	<!DOCTYPE html>
 	<html xmlns="http://www.w3.org/1999/xhtml">
 	<head>
 	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
@@ -181,7 +186,7 @@ function wp_maintenance() {
  *
  * @access private
  * @since 0.71
- * @global int $timestart Seconds and Microseconds added together from when function is called.
+ * @global int $timestart Seconds and microseconds added together from when function is called.
  * @return bool Always returns true.
  */
 function timer_start() {
@@ -205,11 +210,11 @@ function timer_start() {
  * timer_stop(1);
  * </code>
  * which will do what the above does. If you need the result, you can assign it to a variable, but
- * most cases, you only need to echo it.
+ * in most cases, you only need to echo it.
  *
  * @since 0.71
- * @global int $timestart Seconds and Microseconds added together from when timer_start() is called
- * @global int $timeend  Seconds and Microseconds added together from when function is called
+ * @global int $timestart Seconds and microseconds added together from when timer_start() is called
+ * @global int $timeend Seconds and microseconds added together from when function is called
  *
  * @param int $display Use '0' or null to not echo anything and 1 to echo the total time
  * @param int $precision The amount of digits from the right of the decimal to display. Default is 3.
@@ -244,9 +249,9 @@ function timer_stop( $display = 0, $precision = 3 ) { // if called like timer_st
  * development environments.
  *
  * When WP_DEBUG_DISPLAY is true, WordPress will force errors to be displayed.
- * WP_DEBUG_DISPLAY defaults to true. Defining it as false prevents WordPress from
- * changing the global configuration setting. (Defining WP_DEBUG_DISPLAY as false
- * will never force errors to be hidden.)
+ * WP_DEBUG_DISPLAY defaults to true. Defining it as null prevents WordPress from
+ * changing the global configuration setting. Defining WP_DEBUG_DISPLAY as false
+ * will force errors to be hidden.
  *
  * When WP_DEBUG_LOG is true, errors will be logged to wp-content/debug.log.
  * WP_DEBUG_LOG defaults to false.
@@ -256,6 +261,8 @@ function timer_stop( $display = 0, $precision = 3 ) { // if called like timer_st
  */
 function wp_debug_mode() {
 	if ( WP_DEBUG ) {
+		// E_DEPRECATED is a core PHP constant in PHP 5.3. Don't define this yourself.
+		// The two statements are equivalent, just one is for 5.3+ and for less than 5.3.
 		if ( defined( 'E_DEPRECATED' ) )
 			error_reporting( E_ALL & ~E_DEPRECATED & ~E_STRICT );
 		else
@@ -263,16 +270,15 @@ function wp_debug_mode() {
 
 		if ( WP_DEBUG_DISPLAY )
 			ini_set( 'display_errors', 1 );
+		elseif ( null !== WP_DEBUG_DISPLAY )
+			ini_set( 'display_errors', 0 );
 
 		if ( WP_DEBUG_LOG ) {
 			ini_set( 'log_errors', 1 );
 			ini_set( 'error_log', WP_CONTENT_DIR . '/debug.log' );
 		}
 	} else {
-		if ( defined( 'E_RECOVERABLE_ERROR' ) )
-			error_reporting( E_CORE_ERROR | E_CORE_WARNING | E_COMPILE_ERROR | E_ERROR | E_WARNING | E_PARSE | E_USER_ERROR | E_USER_WARNING | E_RECOVERABLE_ERROR );
-		else
-			error_reporting( E_CORE_ERROR | E_CORE_WARNING | E_COMPILE_ERROR | E_ERROR | E_WARNING | E_PARSE | E_USER_ERROR | E_USER_WARNING );
+		error_reporting( E_CORE_ERROR | E_CORE_WARNING | E_COMPILE_ERROR | E_ERROR | E_WARNING | E_PARSE | E_USER_ERROR | E_USER_WARNING | E_RECOVERABLE_ERROR );
 	}
 }
 
@@ -281,8 +287,10 @@ function wp_debug_mode() {
  *
  * To set directory manually, define <code>WP_LANG_DIR</code> in wp-config.php.
  *
- * First looks for language folder in WP_CONTENT_DIR and uses that folder if it
- * exists. Or it uses the "languages" folder in WPINC.
+ * If the language directory exists within WP_CONTENT_DIR, that is used.
+ * Otherwise if the language directory exists within WPINC, that's used.
+ * Finally, if neither of the preceding directories are found,
+ * WP_CONTENT_DIR/languages is used.
  *
  * The WP_LANG_DIR constant was introduced in 2.1.0.
  *
@@ -291,7 +299,7 @@ function wp_debug_mode() {
  */
 function wp_set_lang_dir() {
 	if ( !defined( 'WP_LANG_DIR' ) ) {
-		if ( file_exists( WP_CONTENT_DIR . '/languages' ) && @is_dir( WP_CONTENT_DIR . '/languages' ) ) {
+		if ( file_exists( WP_CONTENT_DIR . '/languages' ) && @is_dir( WP_CONTENT_DIR . '/languages' ) || !@is_dir(ABSPATH . WPINC . '/languages') ) {
 			define( 'WP_LANG_DIR', WP_CONTENT_DIR . '/languages' ); // no leading slash, no trailing slash, full path, not relative to ABSPATH
 			if ( !defined( 'LANGDIR' ) ) {
 				// Old static relative path maintained for limited backwards compatibility - won't work in some cases
@@ -307,6 +315,29 @@ function wp_set_lang_dir() {
 	}
 }
 
+/**
+ * Load the correct database class file.
+ *
+ * This function is used to load the database class file either at runtime or by
+ * wp-admin/setup-config.php. We must globalize $wpdb to ensure that it is
+ * defined globally by the inline code in wp-db.php.
+ *
+ * @since 2.5.0
+ * @global $wpdb WordPress Database Object
+ */
+function require_wp_db() {
+	global $wpdb;
+
+	require_once( ABSPATH . WPINC . '/wp-db.php' );
+	if ( file_exists( WP_CONTENT_DIR . '/db.php' ) )
+		require_once( WP_CONTENT_DIR . '/db.php' );
+
+	if ( isset( $wpdb ) )
+		return;
+
+	$wpdb = new wpdb( DB_USER, DB_PASSWORD, DB_NAME, DB_HOST );
+}
+
 /**
  * Sets the database table prefix and the format specifiers for database table columns.
  *
@@ -365,7 +396,7 @@ function wp_start_object_cache() {
 	} else if ( !$_wp_using_ext_object_cache && file_exists( WP_CONTENT_DIR . '/object-cache.php' ) ) {
 		// Sometimes advanced-cache.php can load object-cache.php before it is loaded here.
 		// This breaks the function_exists check above and can result in $_wp_using_ext_object_cache
-		// being set incorrectly.  Double check if an external cache exists.
+		// being set incorrectly. Double check if an external cache exists.
 		$_wp_using_ext_object_cache = true;
 	}
 
@@ -378,7 +409,7 @@ function wp_start_object_cache() {
 		wp_cache_init();
 
 	if ( function_exists( 'wp_cache_add_global_groups' ) ) {
-		wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'site-transient', 'site-options', 'site-lookup', 'blog-lookup', 'blog-details', 'rss', 'global-posts' ) );
+		wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'user_meta', 'site-transient', 'site-options', 'site-lookup', 'blog-lookup', 'blog-details', 'rss', 'global-posts' ) );
 		wp_cache_add_non_persistent_groups( array( 'comment', 'counts', 'plugins' ) );
 	}
 }
@@ -396,12 +427,9 @@ function wp_not_installed() {
 		if ( ! is_blog_installed() && ! defined( 'WP_INSTALLING' ) )
 			wp_die( __( 'The site you have requested is not installed properly. Please contact the system administrator.' ) );
 	} elseif ( ! is_blog_installed() && false === strpos( $_SERVER['PHP_SELF'], 'install.php' ) && !defined( 'WP_INSTALLING' ) ) {
-		if ( defined( 'WP_SITEURL' ) )
-			$link = WP_SITEURL . '/wp-admin/install.php';
-		elseif ( false !== strpos( $_SERVER['PHP_SELF'], 'wp-admin' ) )
-			$link = preg_replace( '|/wp-admin/?.*?$|', '/', $_SERVER['PHP_SELF'] ) . 'wp-admin/install.php';
-		else
-			$link = preg_replace( '|/[^/]+?$|', '/', $_SERVER['PHP_SELF'] ) . 'wp-admin/install.php';
+
+		$link = wp_guess_url() . '/wp-admin/install.php';
+
 		require( ABSPATH . WPINC . '/kses.php' );
 		require( ABSPATH . WPINC . '/pluggable.php' );
 		require( ABSPATH . WPINC . '/formatting.php' );
@@ -452,15 +480,6 @@ function wp_get_active_and_valid_plugins() {
 	$plugins = array();
 	$active_plugins = (array) get_option( 'active_plugins', array() );
 
-	// Get active network plugins
-	if ( is_multisite() ) {
-		$active_sitewide_plugins = (array) get_site_option( 'active_sitewide_plugins', array() );
-		if ( !empty($active_sitewide_plugins) ) {
-			$active_plugins = array_merge( $active_plugins, array_keys( $active_sitewide_plugins ) );
-			sort( $active_plugins );
-		}
-	}
-
 	// Check for hacks file if the option is enabled
 	if ( get_option( 'hack_file' ) && file_exists( ABSPATH . 'my-hacks.php' ) ) {
 		_deprecated_file( 'my-hacks.php', '1.5' );
@@ -470,10 +489,14 @@ function wp_get_active_and_valid_plugins() {
 	if ( empty( $active_plugins ) || defined( 'WP_INSTALLING' ) )
 		return $plugins;
 
+	$network_plugins = is_multisite() ? wp_get_active_network_plugins() : false;
+
 	foreach ( $active_plugins as $plugin ) {
 		if ( ! validate_file( $plugin ) // $plugin must validate as file
 			&& '.php' == substr( $plugin, -4 ) // $plugin must end with '.php'
 			&& file_exists( WP_PLUGIN_DIR . '/' . $plugin ) // $plugin must exist
+			// not already included as a network plugin
+			&& ( ! $network_plugins || ! in_array( WP_PLUGIN_DIR . '/' . $plugin, $network_plugins ) )
 			)
 		$plugins[] = WP_PLUGIN_DIR . '/' . $plugin;
 	}
@@ -537,23 +560,20 @@ function shutdown_action_hook() {
 /**
  * Copy an object.
  *
- * Returns a cloned copy of an object.
- *
  * @since 2.7.0
+ * @deprecated 3.2
  *
  * @param object $object The object to clone
  * @return object The cloned object
  */
-function wp_clone( $object ) {
-	static $can_clone;
-	if ( !isset( $can_clone ) )
-		$can_clone = version_compare( phpversion(), '5.0', '>=' );
 
-	return $can_clone ? clone( $object ) : $object;
+function wp_clone( $object ) {
+	// Use parens for clone to accommodate PHP 4.  See #17880
+	return clone( $object );
 }
 
 /**
- * Whether the current request is in WordPress admin Panel
+ * Whether the current request is for a network or blog admin page
  *
  * Does not inform on whether the user is an admin! Use capability checks to
  * tell if the user should be accessing a section or not.
@@ -568,6 +588,54 @@ function is_admin() {
 	return false;
 }
 
+/**
+ * Whether the current request is for a blog admin screen /wp-admin/
+ *
+ * Does not inform on whether the user is a blog admin! Use capability checks to
+ * tell if the user should be accessing a section or not.
+ *
+ * @since 3.1.0
+ *
+ * @return bool True if inside WordPress network administration pages.
+ */
+function is_blog_admin() {
+	if ( defined( 'WP_BLOG_ADMIN' ) )
+		return WP_BLOG_ADMIN;
+	return false;
+}
+
+/**
+ * Whether the current request is for a network admin screen /wp-admin/network/
+ *
+ * Does not inform on whether the user is a network admin! Use capability checks to
+ * tell if the user should be accessing a section or not.
+ *
+ * @since 3.1.0
+ *
+ * @return bool True if inside WordPress network administration pages.
+ */
+function is_network_admin() {
+	if ( defined( 'WP_NETWORK_ADMIN' ) )
+		return WP_NETWORK_ADMIN;
+	return false;
+}
+
+/**
+ * Whether the current request is for a user admin screen /wp-admin/user/
+ *
+ * Does not inform on whether the user is an admin! Use capability checks to
+ * tell if the user should be accessing a section or not.
+ *
+ * @since 3.1.0
+ *
+ * @return bool True if inside WordPress user administration pages.
+ */
+function is_user_admin() {
+	if ( defined( 'WP_USER_ADMIN' ) )
+		return WP_USER_ADMIN;
+	return false;
+}
+
 /**
  * Whether Multisite support is enabled
  *
@@ -579,7 +647,7 @@ function is_multisite() {
 	if ( defined( 'MULTISITE' ) )
 		return MULTISITE;
 
-	if ( defined( 'VHOST' ) || defined( 'SUNRISE' ) )
+	if ( defined( 'SUBDOMAIN_INSTALL' ) || defined( 'VHOST' ) || defined( 'SUNRISE' ) )
 		return true;
 
 	return false;
