@@ -26,7 +26,7 @@
  * @param bool $unique Optional, default is false.  Whether the specified metadata key should be
  * 		unique for the object.  If true, and the object already has a value for the specified
  * 		metadata key, no change will be made
- * @return bool True on successful update, false on failure.
+ * @return bool The meta ID on successful update, false on failure.
  */
 function add_metadata($meta_type, $object_id, $meta_key, $meta_value, $unique = false) {
 	if ( !$meta_type || !$meta_key )
@@ -45,10 +45,11 @@ function add_metadata($meta_type, $object_id, $meta_key, $meta_value, $unique =
 	// expected_slashed ($meta_key)
 	$meta_key = stripslashes($meta_key);
 	$meta_value = stripslashes_deep($meta_value);
+	$meta_value = sanitize_meta( $meta_key, $meta_value, $meta_type );
 
 	$check = apply_filters( "add_{$meta_type}_metadata", null, $object_id, $meta_key, $meta_value, $unique );
 	if ( null !== $check )
-		return (bool) $check;
+		return $check;
 
 	if ( $unique && $wpdb->get_var( $wpdb->prepare(
 		"SELECT COUNT(*) FROM $table WHERE meta_key = %s AND $column = %d",
@@ -60,20 +61,22 @@ function add_metadata($meta_type, $object_id, $meta_key, $meta_value, $unique =
 
 	do_action( "add_{$meta_type}_meta", $object_id, $meta_key, $_meta_value );
 
-	$wpdb->insert( $table, array(
+	$result = $wpdb->insert( $table, array(
 		$column => $object_id,
 		'meta_key' => $meta_key,
 		'meta_value' => $meta_value
 	) );
 
+	if ( ! $result )
+		return false;
+
+	$mid = (int) $wpdb->insert_id;
+
 	wp_cache_delete($object_id, $meta_type . '_meta');
-	// users cache stores usermeta that must be cleared.
-	if ( 'user' == $meta_type )
-		clean_user_cache($object_id);
 
-	do_action( "added_{$meta_type}_meta", $wpdb->insert_id, $object_id, $meta_key, $_meta_value );
+	do_action( "added_{$meta_type}_meta", $mid, $object_id, $meta_key, $_meta_value );
 
-	return true;
+	return $mid;
 }
 
 /**
@@ -112,14 +115,16 @@ function update_metadata($meta_type, $object_id, $meta_key, $meta_value, $prev_v
 
 	// expected_slashed ($meta_key)
 	$meta_key = stripslashes($meta_key);
+	$passed_value = $meta_value;
 	$meta_value = stripslashes_deep($meta_value);
+	$meta_value = sanitize_meta( $meta_key, $meta_value, $meta_type );
 
 	$check = apply_filters( "update_{$meta_type}_metadata", null, $object_id, $meta_key, $meta_value, $prev_value );
 	if ( null !== $check )
 		return (bool) $check;
 
 	if ( ! $meta_id = $wpdb->get_var( $wpdb->prepare( "SELECT $id_column FROM $table WHERE meta_key = %s AND $column = %d", $meta_key, $object_id ) ) )
-		return add_metadata($meta_type, $object_id, $meta_key, $meta_value);
+		return add_metadata($meta_type, $object_id, $meta_key, $passed_value);
 
 	// Compare existing value to new value if no prev value given and the key exists only once.
 	if ( empty($prev_value) ) {
@@ -143,14 +148,18 @@ function update_metadata($meta_type, $object_id, $meta_key, $meta_value, $prev_v
 
 	do_action( "update_{$meta_type}_meta", $meta_id, $object_id, $meta_key, $_meta_value );
 
+	if ( 'post' == $meta_type )
+		do_action( 'update_postmeta', $meta_id, $object_id, $meta_key, $meta_value );
+
 	$wpdb->update( $table, $data, $where );
+
 	wp_cache_delete($object_id, $meta_type . '_meta');
-	// users cache stores usermeta that must be cleared.
-	if ( 'user' == $meta_type )
-		clean_user_cache($object_id);
 
 	do_action( "updated_{$meta_type}_meta", $meta_id, $object_id, $meta_key, $_meta_value );
 
+	if ( 'post' == $meta_type )
+		do_action( 'updated_postmeta', $meta_id, $object_id, $meta_key, $meta_value );
+
 	return true;
 }
 
@@ -209,8 +218,14 @@ function delete_metadata($meta_type, $object_id, $meta_key, $meta_value = '', $d
 	if ( !count( $meta_ids ) )
 		return false;
 
+	if ( $delete_all )
+		$object_ids = $wpdb->get_col( $wpdb->prepare( "SELECT $type_column FROM $table WHERE meta_key = %s", $meta_key ) );
+
 	do_action( "delete_{$meta_type}_meta", $meta_ids, $object_id, $meta_key, $_meta_value );
 
+	if ( 'post' == $meta_type )
+		do_action( 'delete_postmeta', $meta_ids );
+
 	$query = "DELETE FROM $table WHERE $id_column IN( " . implode( ',', $meta_ids ) . " )";
 
 	$count = $wpdb->query($query);
@@ -218,13 +233,19 @@ function delete_metadata($meta_type, $object_id, $meta_key, $meta_value = '', $d
 	if ( !$count )
 		return false;
 
-	wp_cache_delete($object_id, $meta_type . '_meta');
-	// users cache stores usermeta that must be cleared.
-	if ( 'user' == $meta_type )
-		clean_user_cache($object_id);
+	if ( $delete_all ) {
+		foreach ( (array) $object_ids as $o_id ) {
+			wp_cache_delete($o_id, $meta_type . '_meta');
+		}
+	} else {
+		wp_cache_delete($object_id, $meta_type . '_meta');
+	}
 
 	do_action( "deleted_{$meta_type}_meta", $meta_ids, $object_id, $meta_key, $_meta_value );
 
+	if ( 'post' == $meta_type )
+		do_action( 'deleted_postmeta', $meta_ids );
+
 	return true;
 }
 
@@ -279,6 +300,213 @@ function get_metadata($meta_type, $object_id, $meta_key = '', $single = false) {
 		return array();
 }
 
+/**
+ * Determine if a meta key is set for a given object
+ *
+ * @since 3.3.0
+ *
+ * @param string $meta_type Type of object metadata is for (e.g., comment, post, or user)
+ * @param int $object_id ID of the object metadata is for
+ * @param string $meta_key Metadata key.
+ * @return boolean true of the key is set, false if not.
+ */
+function metadata_exists( $meta_type, $object_id, $meta_key ) {
+	if ( ! $meta_type )
+		return false;
+
+	if ( ! $object_id = absint( $object_id ) )
+		return false;
+
+	$check = apply_filters( "get_{$meta_type}_metadata", null, $object_id, $meta_key, true );
+	if ( null !== $check )
+		return true;
+
+	$meta_cache = wp_cache_get( $object_id, $meta_type . '_meta' );
+
+	if ( !$meta_cache ) {
+		$meta_cache = update_meta_cache( $meta_type, array( $object_id ) );
+		$meta_cache = $meta_cache[$object_id];
+	}
+
+	if ( isset( $meta_cache[ $meta_key ] ) )
+		return true;
+
+	return false;
+}
+
+/**
+ * Get meta data by meta ID
+ *
+ * @since 3.3.0
+ *
+ * @param string $meta_type Type of object metadata is for (e.g., comment, post, or user)
+ * @param int $meta_id ID for a specific meta row
+ * @return object Meta object or false.
+ */
+function get_metadata_by_mid( $meta_type, $meta_id ) {
+	global $wpdb;
+
+	if ( ! $meta_type )
+		return false;
+
+	if ( !$meta_id = absint( $meta_id ) )
+		return false;
+
+	if ( ! $table = _get_meta_table($meta_type) )
+		return false;
+
+	$id_column = ( 'user' == $meta_type ) ? 'umeta_id' : 'meta_id';
+
+	$meta = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $table WHERE $id_column = %d", $meta_id ) );
+
+	if ( empty( $meta ) )
+		return false;
+
+	if ( isset( $meta->meta_value ) )
+		$meta->meta_value = maybe_unserialize( $meta->meta_value );
+
+	return $meta;
+}
+
+/**
+ * Update meta data by meta ID
+ *
+ * @since 3.3.0
+ *
+ * @uses get_metadata_by_mid() Calls get_metadata_by_mid() to fetch the meta key, value
+ *		and object_id of the given meta_id.
+ *
+ * @param string $meta_type Type of object metadata is for (e.g., comment, post, or user)
+ * @param int $meta_id ID for a specific meta row
+ * @param string $meta_value Metadata value
+ * @param string $meta_key Optional, you can provide a meta key to update it
+ * @return bool True on successful update, false on failure.
+ */
+function update_metadata_by_mid( $meta_type, $meta_id, $meta_value, $meta_key = false ) {
+	global $wpdb;
+
+	// Make sure everything is valid.
+	if ( ! $meta_type )
+		return false;
+
+	if ( ! $meta_id = absint( $meta_id ) )
+		return false;
+
+	if ( ! $table = _get_meta_table( $meta_type ) )
+		return false;
+
+	$column = esc_sql($meta_type . '_id');
+	$id_column = 'user' == $meta_type ? 'umeta_id' : 'meta_id';
+
+	// Fetch the meta and go on if it's found.
+	if ( $meta = get_metadata_by_mid( $meta_type, $meta_id ) ) {
+		$original_key = $meta->meta_key;
+		$original_value = $meta->meta_value;
+		$object_id = $meta->{$column};
+
+		// If a new meta_key (last parameter) was specified, change the meta key,
+		// otherwise use the original key in the update statement.
+		if ( false === $meta_key ) {
+			$meta_key = $original_key;
+		} elseif ( ! is_string( $meta_key ) ) {
+			return false;
+		}
+
+		// Sanitize the meta
+		$_meta_value = $meta_value;
+		$meta_value = sanitize_meta( $meta_key, $meta_value, $meta_type );
+		$meta_value = maybe_serialize( $meta_value );
+
+		// Format the data query arguments.
+		$data = array(
+			'meta_key' => $meta_key,
+			'meta_value' => $meta_value
+		);
+
+		// Format the where query arguments.
+		$where = array();
+		$where[$id_column] = $meta_id;
+
+		do_action( "update_{$meta_type}_meta", $meta_id, $object_id, $meta_key, $_meta_value );
+
+		if ( 'post' == $meta_type )
+			do_action( 'update_postmeta', $meta_id, $object_id, $meta_key, $meta_value );
+
+		// Run the update query, all fields in $data are %s, $where is a %d.
+		$result = (bool) $wpdb->update( $table, $data, $where, '%s', '%d' );
+
+		// Clear the caches.
+		wp_cache_delete($object_id, $meta_type . '_meta');
+
+		do_action( "updated_{$meta_type}_meta", $meta_id, $object_id, $meta_key, $_meta_value );
+
+		if ( 'post' == $meta_type )
+			do_action( 'updated_postmeta', $meta_id, $object_id, $meta_key, $meta_value );
+
+		return $result;
+	}
+
+	// And if the meta was not found.
+	return false;
+}
+
+/**
+ * Delete meta data by meta ID
+ *
+ * @since 3.3.0
+ *
+ * @uses get_metadata_by_mid() Calls get_metadata_by_mid() to fetch the meta key, value
+ *		and object_id of the given meta_id.
+ *
+ * @param string $meta_type Type of object metadata is for (e.g., comment, post, or user)
+ * @param int $meta_id ID for a specific meta row
+ * @return bool True on successful delete, false on failure.
+ */
+function delete_metadata_by_mid( $meta_type, $meta_id ) {
+	global $wpdb;
+
+	// Make sure everything is valid.
+	if ( ! $meta_type )
+		return false;
+
+	if ( ! $meta_id = absint( $meta_id ) )
+		return false;
+
+	if ( ! $table = _get_meta_table( $meta_type ) )
+		return false;
+
+	// object and id columns
+	$column = esc_sql($meta_type . '_id');
+	$id_column = 'user' == $meta_type ? 'umeta_id' : 'meta_id';
+
+	// Fetch the meta and go on if it's found.
+	if ( $meta = get_metadata_by_mid( $meta_type, $meta_id ) ) {
+		$object_id = $meta->{$column};
+
+		do_action( "delete_{$meta_type}_meta", (array) $meta_id, $object_id, $meta->meta_key, $meta->meta_value );
+
+		if ( 'post' == $meta_type )
+			do_action( 'delete_postmeta', $meta_id );
+
+		// Run the query, will return true if deleted, false otherwise
+		$result = (bool) $wpdb->query( $wpdb->prepare( "DELETE FROM $table WHERE $id_column = %d LIMIT 1;", $meta_id ) );
+
+		// Clear the caches.
+		wp_cache_delete($object_id, $meta_type . '_meta');
+
+		do_action( "deleted_{$meta_type}_meta", (array) $meta_id, $object_id, $meta->meta_key, $meta->meta_value );
+
+		if ( 'post' == $meta_type )
+			do_action( 'deleted_postmeta', $meta_id );
+
+		return $result;
+
+	}
+
+	// Meta id was not found.
+	return false;
+}
+
 /**
  * Update the metadata cache for the specified objects.
  *
@@ -355,118 +583,209 @@ function update_meta_cache($meta_type, $object_ids) {
 /**
  * Given a meta query, generates SQL clauses to be appended to a main query
  *
- * @since 3.1.0
- * @access private
+ * @since 3.2.0
  *
- * @param array $meta_query List of metadata queries. A single query is an associative array:
- * - 'key' string The meta key
- * - 'value' string|array The meta value
- * - 'compare' (optional) string How to compare the key to the value.
- *		Possible values: '=', '!=', '>', '>=', '<', '<=', 'LIKE', 'NOT LIKE', 'IN', 'NOT IN', 'BETWEEN', 'NOT BETWEEN'.
- *		Default: '='
- * - 'type' string (optional) The type of the value.
- *		Possible values: 'NUMERIC', 'BINARY', 'CHAR', 'DATE', 'DATETIME', 'DECIMAL', 'SIGNED', 'TIME', 'UNSIGNED'.
- *		Default: 'CHAR'
+ * @see WP_Meta_Query
  *
+ * @param array $meta_query A meta query
  * @param string $type Type of meta
  * @param string $primary_table
  * @param string $primary_id_column
  * @param object $context (optional) The main query object
  * @return array( 'join' => $join_sql, 'where' => $where_sql )
  */
-function _get_meta_sql( $meta_query, $type, $primary_table, $primary_id_column, $context = null ) {
-	global $wpdb;
-
-	if ( ! $meta_table = _get_meta_table( $type ) )
-		return false;
-
-	$meta_id_column = esc_sql( $type . '_id' );
-
-	$join = '';
-	$where = '';
-	$i = 0;
-	foreach ( $meta_query as $q ) {
-		$meta_key = isset( $q['key'] ) ? trim( $q['key'] ) : '';
-		$meta_value = isset( $q['value'] ) ? $q['value'] : '';
-		$meta_compare = isset( $q['compare'] ) ? strtoupper( $q['compare'] ) : '=';
-		$meta_type = isset( $q['type'] ) ? strtoupper( $q['type'] ) : 'CHAR';
-
-		if ( ! in_array( $meta_compare, array( '=', '!=', '>', '>=', '<', '<=', 'LIKE', 'NOT LIKE', 'IN', 'NOT IN', 'BETWEEN', 'NOT BETWEEN' ) ) )
-			$meta_compare = '=';
-
-		if ( 'NUMERIC' == $meta_type )
-			$meta_type = 'SIGNED';
-		elseif ( ! in_array( $meta_type, array( 'BINARY', 'CHAR', 'DATE', 'DATETIME', 'DECIMAL', 'SIGNED', 'TIME', 'UNSIGNED' ) ) )
-			$meta_type = 'CHAR';
-
-		if ( empty( $meta_key ) && empty( $meta_value ) )
-			continue;
-
-		$alias = $i ? 'mt' . $i : $meta_table;
-
-		$join .= "\nINNER JOIN $meta_table";
-		$join .= $i ? " AS $alias" : '';
-		$join .= " ON ($primary_table.$primary_id_column = $alias.$meta_id_column)";
-
-		$i++;
-
-		if ( !empty( $meta_key ) )
-			$where .= $wpdb->prepare( " AND $alias.meta_key = %s", $meta_key );
-
-		if ( in_array( $meta_compare, array( 'IN', 'NOT IN', 'BETWEEN', 'NOT BETWEEN' ) ) ) {
-			if ( ! is_array( $meta_value ) )
-				$meta_value = preg_split( '/[,\s]+/', $meta_value );
-		} else {
-			$meta_value = trim( $meta_value );
-		}
-
-		if ( empty( $meta_value ) )
-			continue;
-
-		if ( 'IN' == substr( $meta_compare, -2) ) {
-			$meta_compare_string = '(' . substr( str_repeat( ',%s', count( $meta_value ) ), 1 ) . ')';
-		} elseif ( 'BETWEEN' == substr( $meta_compare, -7) ) {
-			$meta_value = array_slice( $meta_value, 0, 2 );
-			$meta_compare_string = '%s AND %s';
-		} elseif ( 'LIKE' == substr( $meta_compare, -4 ) ) {
-			$meta_value = '%' . like_escape( $meta_value ) . '%';
-			$meta_compare_string = '%s';
-		} else {
-			$meta_compare_string = '%s';
-		}
-
-		// @todo Temporary hack to support empty values. Do not use outside of core.
-		if ( '_wp_zero_value' == $meta_value )
-			$meta_value = 0;
-
-		$where .= $wpdb->prepare( " AND CAST($alias.meta_value AS {$meta_type}) {$meta_compare} {$meta_compare_string}", $meta_value );
-	}
-
-	return apply_filters_ref_array( 'get_meta_sql', array( compact( 'join', 'where' ), $meta_query, $type, $primary_table, $primary_id_column, &$context ) );
+function get_meta_sql( $meta_query, $type, $primary_table, $primary_id_column, $context = null ) {
+	$meta_query_obj = new WP_Meta_Query( $meta_query );
+	return $meta_query_obj->get_sql( $type, $primary_table, $primary_id_column, $context );
 }
 
 /**
- * Populates the $meta_query property
+ * Container class for a multiple metadata query
  *
- * @access private
- * @since 3.1.0
- *
- * @param array $qv The query variables
+ * @since 3.2.0
  */
-function _parse_meta_query( &$qv ) {
-	$meta_query = array();
+class WP_Meta_Query {
+	/**
+	* List of metadata queries. A single query is an associative array:
+	* - 'key' string The meta key
+	* - 'value' string|array The meta value
+	* - 'compare' (optional) string How to compare the key to the value.
+	*              Possible values: '=', '!=', '>', '>=', '<', '<=', 'LIKE', 'NOT LIKE', 'IN', 'NOT IN', 'BETWEEN', 'NOT BETWEEN'.
+	*              Default: '='
+	* - 'type' string (optional) The type of the value.
+	*              Possible values: 'NUMERIC', 'BINARY', 'CHAR', 'DATE', 'DATETIME', 'DECIMAL', 'SIGNED', 'TIME', 'UNSIGNED'.
+	*              Default: 'CHAR'
+	*
+	* @since 3.2.0
+	* @access public
+	* @var array
+	*/
+	public $queries = array();
 
-	// Simple query needs to be first for orderby=meta_value to work correctly
-	foreach ( array( 'key', 'value', 'compare', 'type' ) as $key ) {
-		if ( !empty( $qv[ "meta_$key" ] ) )
-			$meta_query[0][ $key ] = $qv[ "meta_$key" ];
+	/**
+	 * The relation between the queries. Can be one of 'AND' or 'OR'.
+	 *
+	 * @since 3.2.0
+	 * @access public
+	 * @var string
+	 */
+	public $relation;
+
+	/**
+	 * Constructor
+	 *
+	 * @param array $meta_query (optional) A meta query
+	 */
+	function __construct( $meta_query = false ) {
+		if ( !$meta_query )
+			return;
+
+		if ( isset( $meta_query['relation'] ) && strtoupper( $meta_query['relation'] ) == 'OR' ) {
+			$this->relation = 'OR';
+		} else {
+			$this->relation = 'AND';
+		}
+
+		$this->queries = array();
+
+		foreach ( $meta_query as $key => $query ) {
+			if ( ! is_array( $query ) )
+				continue;
+
+			$this->queries[] = $query;
+		}
 	}
 
-	if ( !empty( $qv['meta_query'] ) && is_array( $qv['meta_query'] ) ) {
-		$meta_query = array_merge( $meta_query, $qv['meta_query'] );
+	/**
+	 * Constructs a meta query based on 'meta_*' query vars
+	 *
+	 * @since 3.2.0
+	 * @access public
+	 *
+	 * @param array $qv The query variables
+	 */
+	function parse_query_vars( $qv ) {
+		$meta_query = array();
+
+		// Simple query needs to be first for orderby=meta_value to work correctly
+		foreach ( array( 'key', 'compare', 'type' ) as $key ) {
+			if ( !empty( $qv[ "meta_$key" ] ) )
+				$meta_query[0][ $key ] = $qv[ "meta_$key" ];
+		}
+
+		// WP_Query sets 'meta_value' = '' by default
+		if ( isset( $qv[ 'meta_value' ] ) && '' !== $qv[ 'meta_value' ] )
+			$meta_query[0]['value'] = $qv[ 'meta_value' ];
+
+		if ( !empty( $qv['meta_query'] ) && is_array( $qv['meta_query'] ) ) {
+			$meta_query = array_merge( $meta_query, $qv['meta_query'] );
+		}
+
+		$this->__construct( $meta_query );
 	}
 
-	$qv['meta_query'] = $meta_query;
+	/**
+	 * Generates SQL clauses to be appended to a main query.
+	 *
+	 * @since 3.2.0
+	 * @access public
+	 *
+	 * @param string $type Type of meta
+	 * @param string $primary_table
+	 * @param string $primary_id_column
+	 * @param object $context (optional) The main query object
+	 * @return array( 'join' => $join_sql, 'where' => $where_sql )
+	 */
+	function get_sql( $type, $primary_table, $primary_id_column, $context = null ) {
+		global $wpdb;
+
+		if ( ! $meta_table = _get_meta_table( $type ) )
+			return false;
+
+		$meta_id_column = esc_sql( $type . '_id' );
+
+		$join = array();
+		$where = array();
+
+		foreach ( $this->queries as $k => $q ) {
+			$meta_key = isset( $q['key'] ) ? trim( $q['key'] ) : '';
+			$meta_type = isset( $q['type'] ) ? strtoupper( $q['type'] ) : 'CHAR';
+
+			if ( 'NUMERIC' == $meta_type )
+				$meta_type = 'SIGNED';
+			elseif ( ! in_array( $meta_type, array( 'BINARY', 'CHAR', 'DATE', 'DATETIME', 'DECIMAL', 'SIGNED', 'TIME', 'UNSIGNED' ) ) )
+				$meta_type = 'CHAR';
+
+			$i = count( $join );
+			$alias = $i ? 'mt' . $i : $meta_table;
+
+			// Set JOIN
+			$join[$i]  = "INNER JOIN $meta_table";
+			$join[$i] .= $i ? " AS $alias" : '';
+			$join[$i] .= " ON ($primary_table.$primary_id_column = $alias.$meta_id_column)";
+
+			$where[$k] = '';
+			if ( !empty( $meta_key ) )
+				$where[$k] = $wpdb->prepare( "$alias.meta_key = %s", $meta_key );
+
+			if ( !isset( $q['value'] ) ) {
+				if ( empty( $where[$k] ) )
+					unset( $join[$i] );
+				continue;
+			}
+
+			$meta_value = $q['value'];
+
+			$meta_compare = is_array( $meta_value ) ? 'IN' : '=';
+			if ( isset( $q['compare'] ) )
+				$meta_compare = strtoupper( $q['compare'] );
+
+			if ( ! in_array( $meta_compare, array( '=', '!=', '>', '>=', '<', '<=', 'LIKE', 'NOT LIKE', 'IN', 'NOT IN', 'BETWEEN', 'NOT BETWEEN' ) ) )
+				$meta_compare = '=';
+
+			if ( in_array( $meta_compare, array( 'IN', 'NOT IN', 'BETWEEN', 'NOT BETWEEN' ) ) ) {
+				if ( ! is_array( $meta_value ) )
+					$meta_value = preg_split( '/[,\s]+/', $meta_value );
+
+				if ( empty( $meta_value ) ) {
+					unset( $join[$i] );
+					continue;
+				}
+			} else {
+				$meta_value = trim( $meta_value );
+			}
+
+			if ( 'IN' == substr( $meta_compare, -2) ) {
+				$meta_compare_string = '(' . substr( str_repeat( ',%s', count( $meta_value ) ), 1 ) . ')';
+			} elseif ( 'BETWEEN' == substr( $meta_compare, -7) ) {
+				$meta_value = array_slice( $meta_value, 0, 2 );
+				$meta_compare_string = '%s AND %s';
+			} elseif ( 'LIKE' == substr( $meta_compare, -4 ) ) {
+				$meta_value = '%' . like_escape( $meta_value ) . '%';
+				$meta_compare_string = '%s';
+			} else {
+				$meta_compare_string = '%s';
+			}
+
+			if ( ! empty( $where[$k] ) )
+				$where[$k] .= ' AND ';
+
+			$where[$k] = ' (' . $where[$k] . $wpdb->prepare( "CAST($alias.meta_value AS {$meta_type}) {$meta_compare} {$meta_compare_string})", $meta_value );
+		}
+
+		$where = array_filter( $where );
+
+		if ( empty( $where ) )
+			$where = '';
+		else
+			$where = ' AND (' . implode( "\n{$this->relation} ", $where ) . ' )';
+
+		$join = implode( "\n", $join );
+		if ( ! empty( $join ) )
+			$join = ' ' . $join;
+
+		return apply_filters_ref_array( 'get_meta_sql', array( compact( 'join', 'where' ), $this->queries, $type, $primary_table, $primary_id_column, $context ) );
+	}
 }
 
 /**
@@ -488,4 +807,59 @@ function _get_meta_table($type) {
 
 	return $wpdb->$table_name;
 }
+
+/**
+ * Determine whether a meta key is protected
+ *
+ * @since 3.1.3
+ *
+ * @param string $meta_key Meta key
+ * @return bool True if the key is protected, false otherwise.
+ */
+function is_protected_meta( $meta_key, $meta_type = null ) {
+	$protected = ( '_' == $meta_key[0] );
+
+	return apply_filters( 'is_protected_meta', $protected, $meta_key, $meta_type );
+}
+
+/**
+ * Sanitize meta value
+ *
+ * @since 3.1.3
+ *
+ * @param string $meta_key Meta key
+ * @param mixed $meta_value Meta value to sanitize
+ * @param string $meta_type Type of meta
+ * @return mixed Sanitized $meta_value
+ */
+function sanitize_meta( $meta_key, $meta_value, $meta_type ) {
+	return apply_filters( "sanitize_{$meta_type}_meta_{$meta_key}", $meta_value, $meta_key, $meta_type );
+}
+
+/**
+ * Register meta key
+ *
+ * @since 3.3.0
+ *
+ * @param string $meta_type Type of meta
+ * @param string $meta_key Meta key
+ * @param string|array $sanitize_callback A function or method to call when sanitizing the value of $meta_key.
+ * @param string|array $auth_callback Optional. A function or method to call when performing edit_post_meta, add_post_meta, and delete_post_meta capability checks.
+ * @param array $args Arguments
+ */
+function register_meta( $meta_type, $meta_key, $sanitize_callback, $auth_callback = null ) {
+	if ( is_callable( $sanitize_callback ) )
+		add_filter( "sanitize_{$meta_type}_meta_{$meta_key}", $sanitize_callback, 10, 3 );
+
+	if ( empty( $auth_callback ) ) {
+		if ( is_protected_meta( $meta_key, $meta_type ) )
+			$auth_callback = '__return_false';
+		else
+			$auth_callback = '__return_true';
+	}
+
+	if ( is_callable( $auth_callback ) )
+		add_filter( "auth_{$meta_type}_meta_{$meta_key}", $auth_callback, 10, 6 );
+}
+
 ?>
