@@ -5,6 +5,8 @@
  *
  * Allows for some configuration in wp-config.php (see default-constants.php)
  *
+ * @internal This file must be parsable by PHP4.
+ *
  * @package WordPress
  */
 
@@ -20,12 +22,15 @@ require( ABSPATH . WPINC . '/load.php' );
 require( ABSPATH . WPINC . '/default-constants.php' );
 require( ABSPATH . WPINC . '/version.php' );
 
-// Set initial default constants including WP_MEMORY_LIMIT, WP_DEBUG, WP_CONTENT_DIR and WP_CACHE.
+// Set initial default constants including WP_MEMORY_LIMIT, WP_MAX_MEMORY_LIMIT, WP_DEBUG, WP_CONTENT_DIR and WP_CACHE.
 wp_initial_constants( );
 
+// Check for the required PHP version and for the MySQL extension or a database drop-in.
+wp_check_php_mysql_versions();
+
 // Disable magic quotes at runtime. Magic quotes are added using wpdb later in wp-settings.php.
-set_magic_quotes_runtime( 0 );
-@ini_set( 'magic_quotes_sybase', 0 );
+@ini_set( 'magic_quotes_runtime', 0 );
+@ini_set( 'magic_quotes_sybase',  0 );
 
 // Set default timezone in PHP 5.
 if ( function_exists( 'date_default_timezone_set' ) )
@@ -40,10 +45,7 @@ unset( $wp_filter, $cache_lastcommentmodified );
 // Standardize $_SERVER variables across setups.
 wp_fix_server_vars();
 
-// Check for the required PHP version and for the MySQL extension or a database drop-in.
-wp_check_php_mysql_versions();
-
-// Check if we have recieved a request due to missing favicon.ico
+// Check if we have received a request due to missing favicon.ico
 wp_favicon_request();
 
 // Check if we're in maintenance mode.
@@ -73,6 +75,7 @@ require( ABSPATH . WPINC . '/plugin.php' );
 require_wp_db();
 
 // Set the database table prefix and the format specifiers for database table columns.
+$GLOBALS['table_prefix'] = $table_prefix;
 wp_set_wpdb_vars();
 
 // Start the WordPress object cache, or an external object cache if the drop-in is present.
@@ -90,6 +93,8 @@ if ( is_multisite() ) {
 	define( 'MULTISITE', false );
 }
 
+register_shutdown_function( 'shutdown_action_hook' );
+
 // Stop most of WordPress from being loaded if we just want the basics.
 if ( SHORTINIT )
 	return false;
@@ -100,6 +105,7 @@ require( ABSPATH . WPINC . '/l10n.php' );
 // Run the installer if WordPress is not installed.
 wp_not_installed();
 
+
 // Load most of WordPress.
 require( ABSPATH . WPINC . '/class-wp-walker.php' );
 require( ABSPATH . WPINC . '/class-wp-ajax-response.php' );
@@ -216,7 +222,7 @@ do_action( 'sanitize_comment_cookies' );
  * @global object $wp_the_query
  * @since 2.0.0
  */
-$wp_the_query =& new WP_Query();
+$wp_the_query = new WP_Query();
 
 /**
  * Holds the reference to @see $wp_the_query
@@ -231,21 +237,21 @@ $wp_query =& $wp_the_query;
  * @global object $wp_rewrite
  * @since 1.5.0
  */
-$wp_rewrite =& new WP_Rewrite();
+$wp_rewrite = new WP_Rewrite();
 
 /**
  * WordPress Object
  * @global object $wp
  * @since 2.0.0
  */
-$wp =& new WP();
+$wp = new WP();
 
 /**
  * WordPress Widget Factory Object
  * @global object $wp_widget_factory
  * @since 2.8.0
  */
-$wp_widget_factory =& new WP_Widget_Factory();
+$GLOBALS['wp_widget_factory'] = new WP_Widget_Factory();
 
 do_action( 'setup_theme' );
 
@@ -258,7 +264,7 @@ load_default_textdomain();
 // Find the blog locale.
 $locale = get_locale();
 $locale_file = WP_LANG_DIR . "/$locale.php";
-if ( is_readable( $locale_file ) )
+if ( ( 0 === validate_file( $locale ) ) && is_readable( $locale_file ) )
 	require( $locale_file );
 unset($locale_file);
 
@@ -270,21 +276,21 @@ require( ABSPATH . WPINC . '/locale.php' );
  * @global object $wp_locale
  * @since 2.1.0
  */
-$wp_locale =& new WP_Locale();
+$GLOBALS['wp_locale'] = new WP_Locale();
 
 // Load the functions for the active theme, for both parent and child theme if applicable.
-if ( TEMPLATEPATH !== STYLESHEETPATH && file_exists( STYLESHEETPATH . '/functions.php' ) )
-	include( STYLESHEETPATH . '/functions.php' );
-if ( file_exists( TEMPLATEPATH . '/functions.php' ) )
-	include( TEMPLATEPATH . '/functions.php' );
+if ( ! defined( 'WP_INSTALLING' ) || 'wp-activate.php' === $pagenow ) {
+	if ( TEMPLATEPATH !== STYLESHEETPATH && file_exists( STYLESHEETPATH . '/functions.php' ) )
+		include( STYLESHEETPATH . '/functions.php' );
+	if ( file_exists( TEMPLATEPATH . '/functions.php' ) )
+		include( TEMPLATEPATH . '/functions.php' );
+}
 
 do_action( 'after_setup_theme' );
 
 // Load any template functions the theme supports.
 require_if_theme_supports( 'post-thumbnails', ABSPATH . WPINC . '/post-thumbnail-template.php' );
 
-register_shutdown_function( 'shutdown_action_hook' );
-
 // Set up current user.
 $wp->init();
 
